'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.GulpTaskBuilder = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _gulpNodemon = require('gulp-nodemon');

var _gulpNodemon2 = _interopRequireDefault(_gulpNodemon);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Gulp task builder used for creating gulp tasks.
 *
 * This simplifies the process of creating tasks that automatically clean itself before starting or watching for changes
 * for it's files.
 */
var GulpTaskBuilder = exports.GulpTaskBuilder = function () {
    /**
     * @param gulp is the gulp executable. For some reason you need to pass it in.
     */
    function GulpTaskBuilder(gulp) {
        _classCallCheck(this, GulpTaskBuilder);

        this._gulp = gulp;
        this._fileTaskSrc = {};
    }

    /**
     * Create a task that when triggered executes a given function
     *
     * @param {string} name of the task for gulp to use
     * @param {string[]} prerequisites task names to be completed before this can be run
     * @param {function} [taskToDo] will be executed after the prerequisite tasks are run
     * @returns {GulpTaskBuilder}
     */


    _createClass(GulpTaskBuilder, [{
        key: 'newTask',
        value: function newTask(name, prerequisites) {
            var taskToDo = arguments.length <= 2 || arguments[2] === undefined ? function () {} : arguments[2];

            this._gulp.task(name, prerequisites, taskToDo);

            return this;
        }

        /**
         * Create a task that will take a given src files, process these files and then possibly move them to a destination.
         * <p>
         *     This also creates a <taskname>-no-clean where we do not first clean the destination folder before executing.
         * </p>
         *
         * @param {string} name of the task for gulp to use
         * @param {string[]} prerequisites task names to be completed before this can be run
         * @param {string[]} src file locator
         * @param {string} [dest] to output the files if necessary
         * @param {function} [taskToDo] will be executed after the prerequisite tasks are run
         * @returns {GulpTaskBuilder}
         */

    }, {
        key: 'newFileTask',
        value: function newFileTask(name, prerequisites, src, dest) {
            var _this = this;

            var taskToDo = arguments.length <= 4 || arguments[4] === undefined ? function (gulpTask) {
                return gulpTask;
            } : arguments[4];

            var fileProcessing = function fileProcessing() {
                var processedSrc = taskToDo(_this._gulp.src(src));
                if (dest) {
                    return processedSrc.pipe(_this._gulp.dest(dest));
                } else {
                    return processedSrc;
                }
            };

            this._gulp.task(name, prerequisites, fileProcessing);

            this._fileTaskSrc[name] = src;

            return this;
        }

        /**
         * Creates a task that when run will watch any of the given tasks src files and re-run them if a change is made.
         * <p>
         * NOTE: this will not monitor any second level task source files.
         * </p>
         *
         * @param {string} name of the task for gulp to use
         * @param {string[]} tasks whose source will be monitored to change to run again
         * @returns {GulpTaskBuilder}
         */

    }, {
        key: 'newWatchableTask',
        value: function newWatchableTask(name, tasks) {
            var _this2 = this;

            this._gulp.task(name + '-prereq', tasks, function () {});

            this.newTask(name, [name + '-prereq'], function () {
                tasks.forEach(function (task) {
                    if (_this2._fileTaskSrc[task]) {
                        console.log("Watching files: ", _this2._fileTaskSrc[task], " and will do '" + task + "' if they change");
                        _this2._gulp.watch(_this2._fileTaskSrc[task], [task]);
                    }
                });
            });

            return this;
        }

        /**
         * Create a gulp task that allows for running a script using nodemon.
         *
         * @param {string} name of the task for gulp to use
         * @param {string[]} tasks which will be run before starting the server as well as when there is a change to the files
         * @param {string[]} srcToWatch would monitor these and re-run the server on changes
         * @param {string} script that would be executed to start the server
         * @returns {GulpTaskBuilder}
         */

    }, {
        key: 'newNodemonTask',
        value: function newNodemonTask(name, tasks, srcToWatch, script) {
            this._gulp.task(name, tasks, function () {
                return (0, _gulpNodemon2.default)({
                    script: script,
                    ext: '*',
                    watch: srcToWatch,
                    tasks: tasks
                });
            });

            return this;
        }

        /**
         * Helper static function for creating the task builder.
         *
         * @param gulp is the gulp executable. For some reason you need to pass it in.
         * @returns {GulpTaskBuilder}
         */

    }], [{
        key: 'newBuilder',
        value: function newBuilder(gulp) {
            return new GulpTaskBuilder(gulp);
        }
    }]);

    return GulpTaskBuilder;
}();
//# sourceMappingURL=index.js.map